% Reactormodel for DBFZ fixed bed reactor / paper appendix
% 2023 Maximilian Heinrich; Andy Gradel
% institute for water and energy management, UAS Hof
% Preliminary stuff
tic
clc
clear
clear global
global pre_1 pre_2 T_nz_t param mn0 ug_o nnn VAR tcount lasttcount VARTRIGGER m0 p_o RG epsp_o M nz T0 HR DR dV dz eps dp lp ug us Acco2 Ecco2 dhcco2 ypcco2 CO_mol tcount dV
[Acco2,Ecco2,dhcco2,ypcco2]=kinetics();
M=molmass();                   %vector for molar masses

for param=1:5   % parameter variation loop
%% Plotting some Variables in the end? yes/no
VAR_PLOT=0; % Plots
EPSP_PLOT=0;
GAS_COMP=1;
GAS_COMP_LAST=0;
CHAR_PLOT=1;   

%% Input
tend=5171; %5171;                %[s]
Vg=1.079;                      %total gas input [NORM-m/h] (DIN 1343: 0C; 101325 Pa)
nz=21;                         %number of element in total

%% Temp-Polynomals from experimental data 
% Extract data of thermocouples:
A=readmatrix('tempfitmatlab.xlsx'); % Tprofile DBFZ-Reactor: Measurement 2022 - 00058
time=A(:,1);
Temp_m1=A(:,2);
Temp_m2=A(:,3);
clear A
pre_1=polyfit(time,Temp_m1,4); % Polynom_1
pre_2=polyfit(time,Temp_m2,2); % Polynom_2
%pre_2=polyfit(time,Temp_m2,4); % Polynom_2

%Create Plot to check Temperature
t=0:tend;  % time array
top=0.105;  %26;   % position end of bed
bot=0;      %15.5; % position start of bed
nsize=(top-bot)/(nz-1);
for i=1:(tend+1)
    T1(i)=pre_1(1)*(i-1)^4 + pre_1(2)*(i-1)^3 + pre_1(3)*(i-1)^2 + pre_1(4)*(i-1) +pre_1(5); %lower sensor
    T2(i)=pre_2(1)*(i-1)^2 + pre_2(2)*(i-1) +pre_2(3); %upper sensor
    %T2(i)=pre_2(1)*(i-1)^4 + pre_2(2)*(i-1)^3 + pre_2(3)*(i-1)^2 + pre_2(4)*(i-1) +pre_2(5); %upper sensor
    y_thermo_1=0.01;%16.5;    % position of lower sensor
    y_thermo_2=0.07;%22.5;    % position of upper sensor
    for n=1:nz
        y(n)=bot+(nsize)*(n-1);
        T_n_t2(i,n)=(T2(i)-T1(i))/(y_thermo_2-y_thermo_1)*(y(n)-y_thermo_1)+T1(i);
    end
end
figure(11)
hold on
plot(time,Temp_m1,linspace(0,tend,tend+1),T1) % data and polynomial
plot(time,Temp_m2,linspace(0,tend,tend+1),T2)
plot(t,T_n_t2(:,nz))                          % top element
plot(t,T_n_t2(:,1))                           % bot element
%     table_T_bot_top=[t',T_n_t2(:,nz),T_n_t2(:,1)];
%     writematrix(table_T_bot_top,'T_bot_top.xlsx')


%% Temperature at beginning
    clear T1 T2
    t=0;
    T1=pre_1(1)*t^4 + pre_1(2)*t^3 + pre_1(3)*t^2 + pre_1(4)*t +pre_1(5);% Thermocouple 1 (lower "cold" T7_8)
    T2=pre_2(1)*t^2 + pre_2(2)*t +pre_2(3);% Thermocouple 2 (upper "warm" T7_7)                 

    y_thermo_1=16.5;   % position of lower sensor in bed [cm]
    y_thermo_2=22.5;   % position of upper sensor in bed [cm]
    top=26;   % position end of bed
    bot=15.5; % position start of bed
    nsize=(top-bot)/(nz-1);
    dT_dz=(y_thermo_2-y_thermo_1)/(T2-T1);
    constant_c=(y_thermo_1*T2-y_thermo_2*T1)/(T2-T1);
    for n=1:nz
        y_thermo_n=bot+(nsize)*(n-1);
        T_n_t(n)=(y_thermo_n-constant_c)/dT_dz;
    end
   
T0=T_n_t(:);

%% for extracting/viewing variables
nnn=0;
VAR=zeros(tend,8);
tcount=0;
lasttcount=0;
VARTRIGGER=0;
CO_mol=zeros(1,tend);
VAR_lw=0.2;          
VAR_count=1;

%% gas mixture:
gas_content=zeros(9,1);
gas_content(5)=0.000;         %O2
gas_content(6)=0.86;          %N2
gas_content(8)=0.14;          %CO2

%% Reactor
HR=0.101;                      %[m]     height of bed
DR=0.103;                      %[m]     diameter (equivalent with thermocouple exception)
dz=HR/(nz-1);                  %[m]     element height
A=DR^2/4*pi;                   %[m]    cross sectional area
eps=0.4602273;                 %[]      bed porosity
epsp=0.88108;                  %[]      particle porosity
epsp_o=epsp;                   %[]      particle porosity t=0
dp=0.004+param*1e-3;           %[m]     particle diameter +param*1e-3
lp=dp;                         %[m]     particle length == particle diameter (no change)

dV=(DR/2)^2*pi*dz;             %[m]    element volume

%% Constants
p_o=101325;                 %[N/m]       pressure in Pascal (norm condition)
RG =8.31446261815324;       %[J/(mol*K)]  gas constant  

%% Writing initial conditions
us=0;
ug=zeros(nz,1);
for i=1:nz
    VgT(i)=Vg*(T0(i)+273.15)/273.15; %[m] volume for initial gas velocity (with T-profile)
    ug(i)=VgT(i)/DR^2*4/pi/eps/3600;
end

y=zeros(nz*13,1);                      %vector for initial condition
for i=1:nz
    rho=density(T0(i)+273.15,M);       %density of species with temperature
    y((i-1)*13+(1:9))=start(eps,epsp,rho,gas_content);
    y((i-1)*13+(10))=T0(i)+273.15;     %initial temp solid
    y((i-1)*13+(11))=T0(i)+273.15;     %initial temp gas
    y((i-1)*13+(12))=T0(i)+273.15;     %initial temp wall
    y((i-1)*13+(13))=epsp;             %initial particle porosity
    ug(i)=ug(i);
end
% Boundaray 
mn0(1:9)=start(eps,epsp,rho,gas_content);
ug_o=ug;
m0=y(1); % [kg/rkm]
m_reactor=y(1)*dV*(nz-1);

%% Solving
k=1000;
t=linspace(0,tend,k);
dt=t(2)-t(1);
options = odeset('RelTol',1e-5,'Abstol',1e-5,'NonNegative',ones(size(y)));
[T,Y]=ode113(@element,t,y,options);
clc

%% Report stuff
Y=Y';
% Mass fraction
for t_n=1:k
    for z_n=2:nz
    CHAR(z_n-1,t_n)=Y((z_n-1)*13+1,t_n)/Y((nz-1)*13+1,1)*100;
    end
end
%timesteps (1 min=60)
zeitschritte=1200;
n_t=round(k/tend*zeitschritte,0);


%% masses and validation
    C_start=sum(Y((1:(nz-1))*13+1,1))*dV; % [kg] initial char mass in reactor
    C_end=sum(Y((1:(nz-1))*13+1,k))*dV;   % [kg] end char mass in reactor
    C_loss=C_start-C_end;                 % [kg] difference
    C_perc(param)=C_end/C_start*100;      % [%] percent left over (with parameter variation)
    
    C_mol_loss=C_loss/M(1);               % [mol] amount of substance decomposed from C
    CO_mol_test=sum(Y((nz-1)*13+7,:))/k/M(7)*dV/dz*ug(nz)*tend/2;% [mol] sum of CO gas during total experiment (assumption ug(nz,k=1000) is uniform during time)   
    CO_mol_reac=sum(Y((1:(nz-1))*13+7,k))*dV;
    
    V_out=dV/dz*eps*ug(nz)*tend/((T0(nz)+273.15)/273.15); %[mN] total gas outlet
    
    disp(num2str(round(C_start,5))+" [kg] initial char mass in reactor")
    disp(num2str(round(C_end,5))+" [kg] end char mass in reactor")
    disp(num2str(round((C_start-C_end),5))+" [kg] difference")
    disp(num2str(round(C_perc(param),5))+" [%] percent left over")
    disp(num2str(round(C_mol_loss,5))+" [mol] amount of substance decomposed from C")
    disp(num2str(round(sum(CO_mol)/2,5))+" [mol] CO amout tracked at outlet")
    disp(num2str(round(CO_mol_test,5))+" [mol] TEST sum of CO at outlet")
    epsp_nz_k=Y((nz-1)*13+13,k);
    epsg=(eps+(1-eps)*epsp);

    
    
%% outlet volume flow [mN]
for i=1:k
    for j=[3 4 5 6 7 8]
    rho=density(20+273.15,M);% T0(nz)
    G(1+j,i)=Y((nz-1)*13+1+j,i)/rho(1+j);
    end
    GP(:,i)=G(:,i)/sum(G(:,i))*100;
end
%in percent [5]
N=GP(6,:);
CO=GP(7,:);
CO2=GP(8,:);
H2=GP(4,:);
H2O=GP(2,:);
O2=GP(5,:);

%% Output Thermocouples
T1=Y(13+10,:)-273.15;
T2=Y(round(nz/2,0)*13+10,:)-273.15;
T3=Y((nz-1)*13+10,:)-273.15;

figure(1)
ax1=subplot(1,1,1);
plot(ax1,T/60,T1,'k',T/60,T2,'c',T/60,T3);
legend(ax1,'bed top','bed mid','bed bot');
title(ax1,['bed temperature against time, Gas = ',num2str(Vg),' m/h']);
xlabel(ax1,'[min]');
ylabel(ax1,'[C]');

uff=0:dz:HR;
figure(1)
ax1=subplot(1,1,1);
plot(ax1,T0,uff(1:21));
legend(ax1,'bed temperature');
title(ax1,'bed temperature against reactor height');
xlabel(ax1,'[C]');
ylabel(ax1,'[m]');
h=gca;
h.FontSize=16;
grid on
h.MinorGridAlpha=0.3;
h.MinorGridLineStyle='-';

%% Plotting gas composition over complete time
if GAS_COMP==1
    figure(6)
    ax5=subplot(1,1,1);
    plot(ax5,T/60,N,'g',T/60,CO,'k',T/60,CO2,'b','LineWidth',2)%,T/60,H2O,'k',T/60,H2,'b',T/60,O2,'y');
    legend(ax5,'N_2','CO','CO_2')%,'Wasser','Wasserstoff','Sauerstoff');
    %title(ax5,'Outlet gas yield');
    ylabel(ax5,'[vol.%]');
    xlabel(ax5,'t [min]');
    h=gca;
    h.FontSize=36;
    h.FontName='Palatino Linotype';
    grid on
    h.GridAlpha=0.3;
    h.XMinorGrid='on';
    h.YMinorGrid='on';
    h.XLim=[0 100];
    h.YLim=[0 100];
    h.XTick=[0 20 40 60 80 100];
    h.YTick=[0 20 40 60 80 100];
    h.TickDir='out';
    h.MinorGridAlpha=0.1;
    h.MinorGridLineStyle='-';
    h.Legend.Location='east';
    h.XMinorTick='off';
    h.YMinorTick='off';
    table_gas_time=[T,N',CO',CO2'];
    writematrix(table_gas_time,'gas_time.xlsx')
end

CO_mean=sum(CO)/length(CO);
CO2_mean=sum(CO2)/length(CO2);

%% Volumefraction per reactor height
    rho=density(20+273.15,M);
    G=zeros(9,nz);
    GP=zeros(9,nz);
    for n_g=1:nz
            for j=[3 4 5 6 7 8]
            G(1+j,n_g)=Y((n_g-1)*13+1+j,k)/rho(1+j);
            end
         GP(:,n_g)=G(:,n_g)/sum(G(:,n_g))*100;
    end
    figure(5)
    sp2=subplot(1,2,1);
    plot(sp2,GP(7,:),fliplr(1:nz)*dz-(nz)*dz,'k:',GP(8,:),fliplr(1:nz)*dz-(nz)*dz,'k')
    ylim([-0.15, 0]);
    legend('CO','CO_2')
    xlabel('Gas composition [vol.-%]')
    ylabel('Height of reactor bed [m]')
    tlt2=title("Gas composition after "+num2str(tend/60)+" min; CO_2-reaction");
    h=gca;
    h.FontSize=18;

    grid on
    h.MinorGridAlpha=0.3;
    h.MinorGridLineStyle='-';
    h.Legend.Location='east';
    
%     table_gas_height=[(fliplr(1:nz)*dz-(nz)*dz);GP(6,:);GP(7,:);GP(8,:)]';
%     writematrix(table_gas_height,'gas_height.xlsx')
%% Plotting epsp if wanted    
if EPSP_PLOT==1
    figure(7)
    hold on
    for i=1:nz-1
        plot(linspace(1,k-1,k),Y(i*13+13,:))
    end
    title('epsilon_p, against timesteps and nz');
    ylabel('[-]');
    xlabel('timesteps k [-]');
    grid on
end
    figure(9)
    hold on
    for i=1:nz-1
        plot(linspace(1,k-1,k),CHAR(i,:))
    end
    title('CHAR-res, against timesteps and nz');
    ylabel('[-]');
    xlabel('timesteps k [-]');
    grid on

%% VAR-Plots (may not work correctly)
    %Plotting variables written in element if wanted
    if VAR_PLOT==1
        figure(8)
        tvar=linspace(1,size(VAR,1),size(VAR,1));
        VAR_name=["ac", "Cco2", "kcco2", "Deff", "PHI", "eta", "kcco2e", "epsg"];
        VAR_unit=["[m/m]","[]","[]","[]","[]","[]","","[1/]" ];

        for i=1:size(VAR,2)
            hold on
            subplot(4,2,i)
            pl=plot(tvar,VAR(:,i));
            if i==1
                tlt3=title("VAR-Plots: dp="+num2str(dp)+"[m]; lp="+num2str(lp)+"[m]; T="+num2str(T0)+"[C]");
            end
        %     if i==2
        %         ylim([0,sum(VAR(:,i))/size(VAR,1)*2]);
        %     end

            pl.LineWidth=VAR_lw;
            xlabel("Time Tend="+num2str(tend)+"[s]")
            ylabel(VAR_name(i)+" "+VAR_unit(i))
            h=gca;
            h.FontSize=16;
            grid on
            h.MinorGridAlpha=0.3;
            h.MinorGridLineStyle='-';

        end
            VAR_legend(VAR_count)=("Parameter Variation "+num2str(i_parameter));   
            VAR_lw=VAR_lw+0.4;          % thicken lines
            VAR_count=VAR_count+1;
    end
    

end
toc


